//+------------------------------------------------------------------+
//|                                                    Sentiment.mq4 |
//|                                       Copyright  2014, tidicofx |
//|                                               tidicofx@yahoo.com |
//|                                         developed for mooniekool |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, tidicofx"
#property link      "tidicofx@yahoo.com"
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 clrDodgerBlue
#property indicator_width1 2
#property indicator_color2 clrLime
#property indicator_width2 2
#property indicator_color3 clrRed
#property indicator_width3 2
#property indicator_level1 60
#property indicator_level2 -60
#property indicator_level3 0
#property indicator_maximum 100
#property indicator_minimum -100
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#include <stderror.mqh>
//+------------------------------------------------------------------+
enum enm_sentiment_source { src_dukascopy, src_oanda };
//+------------------------------------------------------------------+
//| Input Parameters Definition                                      |
//+------------------------------------------------------------------+
extern enm_sentiment_source src = src_dukascopy;
extern int gmt_offset = 0;
//+------------------------------------------------------------------+
//| Local Parameters Definition                                      |
//+------------------------------------------------------------------+
string lbl = "Sentiment.";
double buyer[], histo[], phisto[], nhisto[];
int offset;
string header[];
datetime gmt_offset_sec;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
{
  //------------------------------------------------------------------
  SetIndexStyle(0, DRAW_LINE); SetIndexBuffer(0, buyer); SetIndexLabel(0, "long");
  SetIndexStyle(1, DRAW_HISTOGRAM); SetIndexBuffer(1, phisto); SetIndexLabel(1, "phisto");
  SetIndexStyle(2, DRAW_HISTOGRAM); SetIndexBuffer(2, nhisto); SetIndexLabel(2, "nhisto");
  SetIndexStyle(3, DRAW_NONE); SetIndexBuffer(3, histo); SetIndexLabel(3, NULL);
  IndicatorDigits(2);
  if (src == src_dukascopy) IndicatorShortName("Sentiment Dukascopy (Long, Long-Short)");
  else IndicatorShortName("Sentiment Oanda (Long, Long-Short)");
  offset = 0;
  gmt_offset_sec = gmt_offset * 60;
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
  //------------------------------------------------------------------
  Comment("");
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,      // size of input time series
                 const int prev_calculated,  // bars handled in previous call
                 const datetime& time[],     // Time
                 const double& open[],       // Open
                 const double& high[],       // High
                 const double& low[],        // Low
                 const double& close[],      // Close
                 const long& tick_volume[],  // Tick Volume
                 const long& volume[],       // Real Volume
                 const int& spread[]         // Spread
)
{
  //------------------------------------------------------------------
  if (prev_calculated == 0) ReadQuotes(time, rates_total);
  ReadQuote(time, rates_total);
  return (rates_total);
  //------------------------------------------------------------------
}
//+------------------------------------------------------------------+
void ReadQuote(const datetime &dt[], int total)
{
  if (offset == 0) return;
  int handle;
  string file;
  ResetLastError();
  if (src == src_dukascopy) file = "Dukascopy.log";
  else file = "Oanda.log";
  handle = FileOpen(file, FILE_CSV|FILE_READ, ",", CP_UTF8);
  if (handle < 0)
  {
    string msg="unable to read log file " + file;
    int err = GetLastError();
    if (err > 0) Comment(msg + "\n" + ErrorDescription(err));
    return;
  }
  else
  {
    string buffer = FileReadString(handle);
    ArrayResize(header, 0);
    if (buffer == "") { FileClose(handle); return; }
    else StringSplit2(buffer, header, ";");
    for (int i = 0; i < ArraySize(header); i++)
      if (header[i] == Symbol()) break;
    if (i == ArraySize(header))
    {
      Comment("no history data found for "+Symbol());
      FileClose(handle);
      return;
    }
    string items[];
    datetime gmt;
    FileSeek(handle, offset, SEEK_SET);
    int first = -1;
    while (!FileIsEnding(handle))
    {
      offset = FileTell(handle);
      buffer = FileReadString(handle);
      if (buffer == "") continue;
      gmt = StrToTime(StringSubstr(buffer, 0, 19));
      gmt = StrToTime(StringSubstr(buffer, 0, 19));
      for (int j = 0; j < total; j++)
        if (gmt > dt[j]-gmt_offset_sec) break;
      if (j == total) continue;
      StringSplit2(buffer, items, ";");
      buyer[j-1] = StrToDouble(items[i]);
      histo[j-1] = buyer[j-1] - (100.00 - buyer[j-1]);
      if (histo[j-1] > 0) { phisto[j-1] = histo[j-1]; nhisto[j-1] = 0; }
      else if (histo[j-1] < 0) { nhisto[j-1] = histo[j-1]; phisto[j-1] = 0; }
      else { phisto[j-1] = 0; nhisto[j-1] = 0; }
      if (first == -1) first = j-1;
    }
    FileClose(handle);
    for (int k = first; k >= 0; k--)
    {
      if (buyer[k] == EMPTY_VALUE)
      {
        buyer[k] = buyer[k+1];
        histo[k] = histo[k+1];
        if (histo[k] > 0) { phisto[k] = phisto[k+1]; nhisto[k] = 0; }
        else { nhisto[k] = nhisto[k+1]; phisto[k] = 0; }
      }
    }
  }
}
//+------------------------------------------------------------------+
void ReadQuotes(const datetime &dt[], int total)
{
  offset = 0;
  int handle;
  string file;
  ResetLastError();
  if (src == src_dukascopy) file = "Dukascopy.log";
  else file = "Oanda.log";
  handle = FileOpen(file, FILE_CSV|FILE_READ, ",", CP_UTF8);
  if (handle < 0)
  {
    string msg="unable to read log file " + file;
    int err = GetLastError();
    if (err > 0) Comment(msg + "\n" + ErrorDescription(err));
    return;
  }
  else
  {
    string buffer = FileReadString(handle);
    ArrayResize(header, 0);
    if (buffer == "") { FileClose(handle); return; }
    else StringSplit2(buffer, header, ";");
    for (int i = 0; i < ArraySize(header); i++)
      if (header[i] == Symbol()) break;
    if (i == ArraySize(header))
    {
      Comment("no history data found for "+Symbol());
      FileClose(handle);
      return;
    }
    string items[];
    datetime gmt;
    int first = -1;
    while (!FileIsEnding(handle))
    {
      offset = FileTell(handle);
      buffer = FileReadString(handle);
      if (buffer == "") continue;
      gmt = StrToTime(StringSubstr(buffer, 0, 19));
      for (int j = 0; j < total; j++)
        if (gmt > dt[j]-gmt_offset_sec) break;
      if (j == total) continue;
      StringSplit2(buffer, items, ";");
      buyer[j-1] = StrToDouble(items[i]);
      histo[j-1] = buyer[j-1] - (100.00 - buyer[j-1]);
      if (histo[j-1] > 0) { phisto[j-1] = histo[j-1]; nhisto[j-1] = 0; }
      else if (histo[j-1] < 0) { nhisto[j-1] = histo[j-1]; phisto[j-1] = 0; }
      else { phisto[j-1] = 0; nhisto[j-1] = 0; }
      if (first == -1) first = j-1;
    }
    FileClose(handle);
    for (int k = first; k >= 0; k--)
    {
      if (buyer[k] == EMPTY_VALUE)
      {
        buyer[k] = buyer[k+1];
        histo[k] = histo[k+1];
        if (histo[k] > 0) { phisto[k] = phisto[k+1]; nhisto[k] = 0; }
        else { nhisto[k] = nhisto[k+1]; phisto[k] = 0; }
      }
    }
  }
}
//+------------------------------------------------------------------+
void StringSplit2(string InputString, string &ResultArray[], string Separator)
{
  ArrayResize(ResultArray, 0);
  int lenSeparator = StringLen(Separator), NewArraySize;
  while (InputString != "") {
    int p = StringFind(InputString, Separator);
    if (p == -1) {
      NewArraySize = ArraySize(ResultArray) + 1;
      ArrayResize(ResultArray, NewArraySize);      
      ResultArray[NewArraySize - 1] = InputString;
      InputString = "";
    } else {
      NewArraySize = ArraySize(ResultArray) + 1;
      ArrayResize(ResultArray, NewArraySize);      
      ResultArray[NewArraySize - 1] = StringSubstr(InputString, 0, p);
      InputString = StringSubstr(InputString, p + lenSeparator);
      if (InputString == "") {
         ArrayResize(ResultArray, NewArraySize + 1);      
         ResultArray[NewArraySize] = "";
      }
    }     
  }
}
//+------------------------------------------------------------------+
